/*****************************************************************************
  Copyright (c) 2003 University of West Bohemia, DCS, Pilsen, Czech Republic

                               21.2.2003

                            C-Sim version 5.1

                               file OQN.C

 Demonstration example - open queuing network.

 Network consists of two serving nodes (FIFO queue and one-channel
 server with exponential pdf of service time, parameters mi1, mi2)
 and from two transaction sources with exponential pdf of interarrival
 time (parameters lambda1, lambda2).

    source1-->queue1-->server1-> departure  (probability 1-p12)
                              |
                              -> queue2     (probability p12)

    source1-->queue2-->server2-> departure  (probability 1-p21)
                              |
                              -> queue1     (probability p21)

 The aim of the model construction is to estimate a mean response
 time (i.e. the mean time of passing the transactions through the
 network - between arrival from source and departure out of network).
 ********************************************************************/

#include <stdlib.h>
#include <stdio.h>

#include "csim.h"
#include "csim_err.h"
#include "csim_rng.h"

#define INIT_SEED 1111

/* Type definitions */
/* Transaction - derived from CSIM_LINK */
typedef struct {
  csim_d_link;
  double t_input;     /* t_input of transaction csim_into network */
} TRANSACTION;

/* Queue - derived from CSIM_HEAD */
typedef struct {
  csim_d_head;
  CSIM_PROCESS *server;    /* pointer to a csim_process serving the queue */
} QUEUE;

/* Server - derived from CSIM_PROCESS */
typedef struct {
  csim_d_process;
  float mi;            /* rate of service, exp. pdf */
  float p;             /* prob. of departure after the service */
  QUEUE *in_queue;     /* pointer to an input queue */
  QUEUE *out_queue;    /* pointer to an output queue */
  long int cnt;        /* counter of serviced transactions */
  double stq;          /* sum of responses of finished transactions */
} SERVER;

/* Function to initialize the instance of SERVER type */
void ini_server (SERVER* par_serv, float par_mi, float par_p,
                 QUEUE* par_queue_in, QUEUE* par_queue_out) {
  par_serv->mi = par_mi;
  par_serv->p  = par_p;
  par_serv->in_queue  = par_queue_in;
  par_serv->out_queue = par_queue_out;
  par_serv->cnt = 0;
  par_serv->stq = 0.0;
}

/* SOURCE - derived from CSIM_PROCESS */
typedef struct {
  csim_d_process;
  float lambda;     /* rate of arrivals - exp. pdf  */
  QUEUE *queue;     /* pointer to queue to enroll new transaction */
} SOURCE;

/* Function to initialize an instance of SOURCE type */
void ini_source (SOURCE* par_src, float par_lambda,
                 QUEUE* par_queue) {
  par_src->lambda = par_lambda;
  par_src->queue  = par_queue;
}

/* global data definitions */

QUEUE *queue1,*queue2;    /* pointers to queue 1 and 2 */
SERVER *server1,*server2; /* pointers to server processes 1 and 2 */
SOURCE *source1,*source2; /* pointers to source processes 1 and 2 */
CSIM_RNG_STATE *rng;      /* pointer to an object of random numbers
                             generator  */
float mi1     = 1.0;      /* service rate of server 1 */
float mi2     = 1.0;      /* service rate of server 2 */
float lambda1 = 0.4;      /* arrival rate from source 1  */
float lambda2 = 0.4;      /* arrival rate from source 2  */
float p12 = 0.5;          /* prob. of trns. passing from 1 to 2 */
float p21 = 0.5;          /* prob. of trns. passing from 2 to 1 */

/* program definition for a csim_process of the SERVER type */
csim_program(SERVER, SERVER_PROG)

  TRANSACTION *p_poin;
   /* temporary (automatic) pointer to a served transaction */

  for (;;) {
    if (!csim_empty((CSIM_HEAD *) my.in_queue)) {
      csim_hold(csim_negexp(rng, my.mi));    /* emulates the service */
      /* serviced transaction stays in my.in_queue */
      p_poin = (TRANSACTION *) csim_first((CSIM_HEAD *) my.in_queue);
      if (csim_draw(rng, my.p) == FALSE) {   /* departure */
        my.cnt ++;                           /* statistics */
        my.stq += csim_time() - p_poin->t_input;
        csim_dispose_link((CSIM_LINK*) p_poin);
                                       /* csim_out of queue & destroy */
      }
      else {  /* passing to another serving node */
        csim_out((CSIM_LINK *) p_poin);
        csim_into((CSIM_LINK *) p_poin, (CSIM_HEAD *) my.out_queue);
        /* awaking of sleeping server */
        if (csim_idle (my.out_queue->server)) {
          csim_activate_at (my.out_queue->server, csim_time());
        }
      }
    }
    else {
      csim_passivate(); /* emulates waiting when no transaction is ready */
    }
  }
csim_end_program


/* program definition for a csim_process of the SOURCE type */
csim_program(SOURCE, SOURCE_PROG)

  TRANSACTION *p_poin;
     /* temporary (automatic) pointer to a generated transaction */

  for (;;) {
    p_poin = csim_new_link (TRANSACTION);
    p_poin->t_input = csim_time();
    csim_into ((CSIM_LINK *) p_poin, (CSIM_HEAD *) my.queue );
    if (csim_idle (my.queue->server)) {
      csim_activate_at (my.queue->server, csim_time());
    }
    csim_hold(csim_negexp(rng, my.lambda));  /* interarrival time */
  }

csim_end_program


/* Function for the model initialization */
CSIM_BOOLEAN init (void) {
 /* csim's data initialization */
  if (csim_init_mem()) return TRUE;

 /* stable objects creating and initializing */
  rng = csim_new_rng(INIT_SEED);
  if (rng == NULL) return TRUE;
  queue1  = csim_new_head (QUEUE);
  if (queue1 == NULL) return TRUE;
  queue2  = csim_new_head (QUEUE);
  if (queue2 == NULL) return TRUE;

  server1 = csim_new_process(SERVER, SERVER_PROG);
  if (server1 == NULL) return TRUE;
  ini_server (server1, mi1, p12, queue1, queue2);
  server2 = csim_new_process(SERVER, SERVER_PROG);
  if (server2 == NULL) return TRUE;
  ini_server (server2, mi2, p21, queue2, queue1);

  source1  = csim_new_process(SOURCE, SOURCE_PROG);
  if (source1 == NULL) return TRUE;
  ini_source (source1, lambda1, queue1);
  source2  = csim_new_process(SOURCE, SOURCE_PROG);
  if (source2 == NULL) return TRUE;
  ini_source (source2, lambda2, queue2);

  queue1->server = (CSIM_PROCESS *) server1;
  queue2->server = (CSIM_PROCESS *) server2;

  /* initial activation of processes */
  if (csim_activate_at ((CSIM_PROCESS *) source1, 0.0))
    return TRUE;
  if (csim_activate_at ((CSIM_PROCESS *) source2, 0.0))
    return TRUE;

  return FALSE;
}

/* main program */

int main(void)
{
  CSIM_ERROR er;
  long int trns_cnt = 0;

  if (init()) {
    csim_error(&er);
    printf("Init: %s\n", csim_error_msg(er.error_code));
    (void) getchar();
    csim_clear_mem();
    return (0);
  }

  printf ("OQN - open queuing network model \n");
  printf ("Number of passing transactions (100 000 should pass): \n");
  while(trns_cnt < 100000) {
    if (csim_step() == FAILURE) {
      csim_error(&er);
      printf("Loop: %s\n", csim_error_msg(er.error_code));
      (void) getchar();
      csim_clear_mem();
      return (0);
    }
    trns_cnt = server1->cnt + server2->cnt;
    printf ("%li\r", trns_cnt);
  }

  printf ("\nExperiment result : mean time of response = %f\n",
    ((server1->stq + server2->stq) / (server1->cnt + server2->cnt)));
  printf ("Mathematical model: mean time of response = 10.00 \n");
  printf ("Push Enter key! \n");

  (void) getchar();
  csim_clear_mem();
  return (0);
}
